﻿Module Module_RollingGraph

    Friend RollingGraph_Scale As Single = 1  ' 1 to 4
    Friend RollingGraph_MeasureUnit As String

    Friend RedrawSinglePointPositionX As Int32
    Friend RollingGraph_OldPositionY As Single = 9999999

    Friend Sub RollingGraph_PlotGraph(ByVal pbox As PictureBox, _
                                      ByVal speed As Int32, _
                                      ByVal value As Single, _
                                      ByVal showCandle As Boolean, _
                                      ByVal maxCandle As Single, _
                                      ByVal minCandle As Single, _
                                      ByVal VerticalDivisionLength As Int32, _
                                      Optional ByVal RedrawSinglePoint As Boolean = False, _
                                      Optional ByVal MaxCount As Boolean = False)

        ' ----------------------------------------- correct by MeasureUnit
        Select Case RollingGraph_MeasureUnit
            Case "Sievert"
                '
            Case "Roentgen"
                value /= 100
                maxCandle /= 100
                minCandle /= 100
            Case "Curie"
                '
            Case "Bequerel"
                value /= 100
                maxCandle /= 100
                minCandle /= 100
        End Select


        Dim startx As Int32 = 1
        Dim RedrawBackground As Boolean = False

        ' ------------------------------------------------------ using a bitmap for best performance
        If pbox.Image Is Nothing OrElse pbox.Image.Width <> pbox.ClientSize.Width OrElse pbox.Image.Height <> pbox.ClientSize.Height Then
            If pbox.ClientSize.Width > 0 AndAlso pbox.ClientSize.Height > 0 Then
                InitPictureboxImage(pbox)
                startx = pbox.Image.Width
                RollingGraph_OldPositionY = 9999999
                RedrawBackground = True
            End If
        End If
        ' ------------------------------------------------------ using "Graphics.FromImage" for best performance
        If pbox.Image Is Nothing Then Return
        Dim g As Graphics = Graphics.FromImage(pbox.Image)

        ' ------------------------------------------------------ init pens
        Dim c1 As Color = Color.DarkGreen
        Dim c2 As Color = Color.FromArgb(240, 255, 235)
        Dim c3 As Color = Color.FromArgb(255, 255, 220)
        Dim c4 As Color = Color.FromArgb(255, 240, 220)
        Dim p1 As Pen = New Pen(c1, 2)
        Dim pCandles As Pen = New Pen(Color.FromArgb(150, 150, 150))

        ' ------------------------------------------------------ init positions
        Dim px As Single
        If RedrawSinglePoint Then
            px = RedrawSinglePointPositionX
        Else
            px = pbox.Image.Width - 1
        End If

        ' ------------------------------------------------------ draw color-areas
        If Not RedrawSinglePoint Then
            Dim p2 As Pen = New Pen(c2)
            Dim p3 As Pen = New Pen(c3)
            Dim p4 As Pen = New Pen(c4)

            Dim py2 As Single = ValueToY(0.01, pbox.Image.Height)
            Dim py3 As Single = ValueToY(1, pbox.Image.Height)
            Dim py4 As Single = ValueToY(10, pbox.Image.Height)
            Dim py5 As Single = ValueToY(10000, pbox.Image.Height)
            If RedrawBackground Then
                g.FillRectangle(New SolidBrush(c2), px - startx, py3, startx, py2 - py3)
                g.FillRectangle(New SolidBrush(c3), px - startx, py4, startx, py3 - py4)
                g.FillRectangle(New SolidBrush(c4), px - startx, py5, startx, py4 - py5)
            Else
                g.DrawLine(p2, px, py2, px, py3)
                g.DrawLine(p3, px, py3, px, py4)
                g.DrawLine(p4, px, py4, px, py5)
            End If
        End If

        ' ------------------------------------------------------ draw candle-stick 
        If showCandle Then
            Dim pyHI As Single = ValueToY(maxCandle, pbox.Image.Height)
            Dim pyLOW As Single = ValueToY(minCandle, pbox.Image.Height)
            g.DrawLine(pCandles, px - 2, pyLOW, px - 2, pyHI)
            g.DrawLine(pCandles, px - 3, pyLOW, px, pyLOW)
            g.DrawLine(pCandles, px - 3, pyHI, px, pyHI)
        End If

        ' ------------------------------------------------------ draw horizontal scale lines and text
        If Not RedrawSinglePoint Then
            Dim j As Single = 0.01
            Do
                For i As Int32 = 1 To 9
                    Dim y As Int32 = ValueToY(i * j, pbox.Image.Height)
                    If i = 1 Then
                        g.DrawLine(Pens.Gray, px - startx, y, px, y)
                    ElseIf i = 5 Then
                        g.DrawLine(Pens.DarkGray, px - startx, y, px, y)
                    Else
                        g.DrawLine(Pens.LightGray, px - startx, y, px, y)
                    End If
                Next
                '
                Dim s As String = ""
                Select Case RollingGraph_MeasureUnit
                    Case "Sievert"
                        s = (Int(j * 100 + 0.05) / 100).ToString()
                    Case "Roentgen"
                        s = (Int(j * 10000 + 0.05) / 100).ToString("0")
                    Case "Curie"
                        s = (Int(j * 100 + 0.05) / 100).ToString()
                    Case "Bequerel"
                        s = (Int(j * 10000 + 0.5) / 100).ToString("0")
                End Select
                '
                Dim f As Font = New Font("Arial", 8)
                Dim ty As Int32 = ValueToY(j, pbox.Image.Height) - 16
                Dim textsize As SizeF = g.MeasureString(s, f)
                g.FillRectangle(Brushes.White, 0, ty, textsize.Width, 16)
                g.DrawString(s, f, Brushes.Black, 0, ty)
                '
                j = j * 10
            Loop Until j >= 10000
        End If

        ' ------------------------------------------------------ draw indicator
        Dim py As Single = ValueToY(value, pbox.Image.Height)
        If Math.Abs(RollingGraph_OldPositionY - py) > 1000 Then RollingGraph_OldPositionY = py
        g.DrawLine(p1, px - 3, RollingGraph_OldPositionY, px - 2, py)
        RollingGraph_OldPositionY = py

        ' ------------------------------------------------------ vertical divisions
        If VerticalDivisionLength > 0 Then
            If VerticalDivisionLength > pbox.Image.Height Then VerticalDivisionLength = pbox.Image.Height
            If VerticalDivisionLength > 16 Then
                ' minutes
                Dim p As New Pen(Color.FromArgb(140, 100, 0), 2)
                g.DrawLine(p, px - 2, pbox.Image.Height - 8, px - 2, pbox.Image.Height - 8 - VerticalDivisionLength)
                g.DrawLine(p, px - 2, 0, px - 2, VerticalDivisionLength)
            Else
                ' ten seconds
                g.DrawLine(Pens.Gray, px - 2, pbox.Image.Height - 8, px - 2, pbox.Image.Height - 8 - VerticalDivisionLength)
                g.DrawLine(Pens.Gray, px - 2, 0, px - 2, VerticalDivisionLength)
            End If
        End If

        ' ------------------------------------------------------ scroll
        Static n As Single
        n += PixelsPerSecond(speed)
        'Select Case speed
        '    Case 10 : n += 10    ' 60 seconds with a 600 pixel window
        '    Case 9 : n += 5      ' 2 min
        '    Case 8 : n += 2      ' 5 mun
        '    Case 7 : n += 1      ' 10 min
        '    Case 6 : n += 0.66F  ' 15 min
        '    Case 5 : n += 0.5F   ' 20 min
        '    Case 4 : n += 0.33F  ' 30 min
        '    Case 3 : n += 0.166F ' 1 h
        '    Case 2 : n += 0.014F ' 12 h
        '    Case 1 : n += 0.007F ' 1 day
        'End Select
        If n > 10 Then
            n -= 10
            If RedrawSinglePoint Then
                RedrawSinglePointPositionX -= 1
            Else
                ImageScrollInPlace(pbox.Image, -1, 0, 0)
                pbox.Invalidate()
            End If
        End If
        ' ------------------------------------------------------ MaxCount indicator
        If MaxCount Then
            g.FillRectangle(Brushes.Yellow, 0, 20, 450, 24)
            g.DrawString("Warning: CPS period near to sensor DeadTime.", New Font("Arial", 12), Brushes.Red, 4, 20)
        End If
    End Sub

    Private Function ValueToY(ByVal value As Single, ByVal height As Single) As Int32
        Dim py As Int32 = 0
        If value > 0 Then py = CInt(height - (8 + (height - 8) * RollingGraph_Scale * 0.0725 * (Math.Log(value) - Math.Log(0.01))))
        Return py
    End Function

    Friend Function PixelsPerSecond(ByVal speed As Single) As Single
        Select Case speed
            Case 10 : Return 10    ' 60 seconds with a 600 pixel window
            Case 9 : Return 5      ' 2 min
            Case 8 : Return 2      ' 5 mun
            Case 7 : Return 1      ' 10 min
            Case 6 : Return 0.66F  ' 15 min
            Case 5 : Return 0.5F   ' 20 min
            Case 4 : Return 0.33F  ' 30 min
            Case 3 : Return 0.166F ' 1 h
            Case 2 : Return 0.014F ' 12 h
            Case 1 : Return 0.007F ' 1 day
        End Select
    End Function


End Module
