﻿Module Module_Utils


    ' ==============================================================================================================
    '   COMBO FUNCTIONS
    ' ==============================================================================================================
    Friend Sub Combo_Init(ByVal combo As ComboBox, ByVal str As String)
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If str = Nothing Then str = ""
        With combo
            .Items.Clear()
            .Items.Add(str)
            .SelectedIndex = 0
        End With
        EventsAreEnabled = old
    End Sub

    Friend Sub Combo_SetIndex_FromString(ByVal combo As ComboBox, ByVal str As String)
        Dim old As Boolean = EventsAreEnabled
        EventsAreEnabled = False
        If str = Nothing Then str = ""
        With combo
            For i As Int32 = 0 To .Items.Count - 1
                If .Items(i).ToString.ToLower = str.ToLower Then
                    .SelectedIndex = i
                    Exit For
                End If
            Next
        End With
        EventsAreEnabled = old
    End Sub

    Friend Function Combo_GetValue(ByVal combo As ComboBox) As String
        If combo.SelectedIndex < 0 Then Return ""
        Return combo.Items(combo.SelectedIndex).ToString()
    End Function

    Friend Sub Combo_SetIndex(ByVal combo As ComboBox, ByVal index As Int32)
        If combo.Items.Count < 1 Then Exit Sub
        If index < 0 Then index = 0
        If index > combo.Items.Count - 1 Then index = combo.Items.Count - 1
        combo.SelectedIndex = index
    End Sub


    ' =======================================================================================================
    '   SMOOTHING
    ' =======================================================================================================
    Friend Sub SmoothValue(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
        If speed > 1 Then speed = 1
        value += (new_value - value) * speed
    End Sub

    Friend Sub SmoothValue_Pow(ByRef value As Single, ByVal new_value As Single, ByVal speed As Single)
        Dim delta As Single = new_value - value
        Dim delta_sgn As Single = Math.Sign(delta)
        Dim delta_abs As Single = Math.Abs(delta)
        Dim delta_pow As Single

        delta_pow = CSng((0.003 * speed * delta_abs) ^ 2)

        If delta_pow >= delta_abs Then
            value = new_value
        Else
            value += delta_sgn * delta_pow
        End If
    End Sub


    ' =======================================================================================================
    '   REDIM PRESERVE BIDIMENSIONAL STRING ARRAY
    ' =======================================================================================================
    Friend Sub RedimPreserve_2D_Array(ByRef ar(,) As String, ByVal rowUpperIdx As Int32, ByVal colUpperIdx As Int32)
        Dim newArray(rowUpperIdx, colUpperIdx) As String
        Dim minRows As Integer = Math.Min(rowUpperIdx + 1, ar.GetLength(0))
        Dim minCols As Integer = Math.Min(colUpperIdx + 1, ar.GetLength(1))
        For i As Int32 = 0 To minRows - 1
            For j As Int32 = 0 To minCols - 1
                newArray(i, j) = ar(i, j)
            Next
        Next
        ar = newArray
    End Sub

    'Friend Sub RedimPreserve_2D_Array(ByRef a(,) As String, ByVal d1 As Int32, ByVal d2 As Int32)
    '    If a.GetUpperBound(1) >= 0 Then
    '        Dim tmpArr(d1, a.GetUpperBound(1)) As String
    '        For i As Int32 = 0 To d1
    '            For j As Int32 = 0 To a.GetUpperBound(1)
    '                tmpArr(i, j) = a(i, j)
    '            Next
    '        Next
    '        ReDim a(d1, d2)
    '        For i As Int32 = 0 To d1
    '            For j As Int32 = 0 To Math.Min(a.GetUpperBound(1), tmpArr.GetUpperBound(1))
    '                a(i, j) = tmpArr(i, j)
    '            Next
    '        Next
    '    Else
    '        ReDim a(d1, d2)
    '    End If
    'End Sub



    ' =======================================================================================================
    '   Round DateTime 
    ' =======================================================================================================
    Friend Function RoundUp(ByVal t As DateTime, ByVal secs As Int32) As DateTime
        Return RoundUp(t, New TimeSpan(0, 0, secs))
    End Function
    Friend Function RoundUp(ByVal dt As DateTime, ByVal ts As TimeSpan) As DateTime
        Dim remainder As Long = dt.Ticks Mod ts.Ticks
        If remainder = 0 Then Return dt
        Return dt.AddTicks(ts.Ticks - remainder)
    End Function
    Friend Function RoundDown(ByVal t As DateTime, ByVal secs As Int32) As DateTime
        Return RoundDown(t, New TimeSpan(0, 0, secs))
    End Function
    Friend Function RoundDown(ByVal dt As DateTime, ByVal ts As TimeSpan) As DateTime
        Dim remainder As Long = dt.Ticks Mod ts.Ticks
        If remainder = 0 Then Return dt
        Return dt.AddTicks(-remainder)
    End Function
    Friend Function RoundToNearest(ByVal t As DateTime, ByVal secs As Int32) As DateTime
        Return RoundToNearest(t, New TimeSpan(0, 0, secs))
    End Function
    Friend Function RoundToNearest(ByVal dt As DateTime, ByVal ts As TimeSpan) As DateTime
        Dim remainder As Long = dt.Ticks Mod ts.Ticks
        If remainder = 0 Then Return dt
        If remainder > ts.Ticks / 2 Then
            Return dt.AddTicks(ts.Ticks - remainder)
        Else
            Return dt.AddTicks(-remainder)
        End If
    End Function

    ' Alignements
    ' 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30  seconds
    ' 1, 2, 3, 4, 5, 6, 10, 12, 15, 20, 30  minutes
    ' 1, 2, 3, 4, 6, 8, 12, 24              hours

    ' =======================================================================================================
    '   CURSOR - HOURGLASS and DEFAULT
    ' =======================================================================================================
    Friend Sub SetCursor_Hourglass()
        Application.UseWaitCursor = True
        Cursor.Current = Cursors.WaitCursor
        Application.DoEvents()
    End Sub
    Friend Sub SetCursor_Default()
        Application.UseWaitCursor = False
        Cursor.Current = Cursors.Default
        Application.DoEvents()
    End Sub

End Module
